@@ -1,6 +1,6 @@
 # Natural Language Toolkit: Applications package
 #
-# Copyright (C) 2001-2017 NLTK Project
+# Copyright (C) 2001-2019 NLTK Project
 # Author: Edward Loper <edloper@gmail.com>
 #         Steven Bird <stevenbird1@gmail.com>
 # URL: <http://nltk.org/>
@@ -25,8 +25,8 @@ try:
     from six.moves import tkinter
 except ImportError:
     import warnings
-    warnings.warn("nltk.app package not loaded "
-                  "(please install Tkinter library).")
+
+    warnings.warn("nltk.app package not loaded " "(please install Tkinter library).")
 else:
     from nltk.app.chartparser_app import app as chartparser
     from nltk.app.chunkparser_app import app as chunkparser
@@ -41,12 +41,15 @@ else:
         from matplotlib import pylab
     except ImportError:
         import warnings
-        warnings.warn("nltk.app.wordfreq not loaded "
-                      "(requires the matplotlib library).")
+
+        warnings.warn(
+            "nltk.app.wordfreq not loaded " "(requires the matplotlib library)."
+        )
     else:
         from nltk.app.wordfreq_app import app as wordfreq
 
 # skip doctests from this package
 def setup_module(module):
     from nose import SkipTest
+
     raise SkipTest("nltk.app examples are not doctests")
