@@ -1,10 +1,12 @@
 # Natural Language Toolkit: Clusterer Utilities
 #
-# Copyright (C) 2001-2015 NLTK Project
+# Copyright (C) 2001-2019 NLTK Project
 # Author: Trevor Cohn <tacohn@cs.mu.oz.au>
+# Contributor: J Richard Snape
 # URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
-from __future__ import print_function, unicode_literals
+from __future__ import print_function, unicode_literals, division
+from abc import abstractmethod
 
 import copy
 from sys import stdout
@@ -18,12 +20,14 @@ except ImportError:
 from nltk.cluster.api import ClusterI
 from nltk.compat import python_2_unicode_compatible
 
+
 class VectorSpaceClusterer(ClusterI):
     """
     Abstract clusterer which takes tokens and maps them into a vector space.
     Optionally performs singular value decomposition to reduce the
     dimensionality.
     """
+
     def __init__(self, normalise=False, svd_dimensions=None):
         """
         :param normalise:       should vectors be normalised to length 1
@@ -46,10 +50,11 @@ class VectorSpaceClusterer(ClusterI):
         # use SVD to reduce the dimensionality
         if self._svd_dimensions and self._svd_dimensions < len(vectors[0]):
             [u, d, vt] = numpy.linalg.svd(numpy.transpose(numpy.array(vectors)))
-            S = d[:self._svd_dimensions] * \
-                numpy.identity(self._svd_dimensions, numpy.float64)
-            T = u[:,:self._svd_dimensions]
-            Dt = vt[:self._svd_dimensions,:]
+            S = d[: self._svd_dimensions] * numpy.identity(
+                self._svd_dimensions, numpy.float64
+            )
+            T = u[:, : self._svd_dimensions]
+            Dt = vt[: self._svd_dimensions, :]
             vectors = numpy.transpose(numpy.dot(S, Dt))
             self._Tt = numpy.transpose(T)
 
@@ -60,11 +65,11 @@ class VectorSpaceClusterer(ClusterI):
         if assign_clusters:
             return [self.classify(vector) for vector in vectors]
 
+    @abstractmethod
     def cluster_vectorspace(self, vectors, trace):
         """
         Finds the clusters using the given set of vectors.
         """
-        raise NotImplementedError()
 
     def classify(self, vector):
         if self._should_normalise:
@@ -74,11 +79,11 @@ class VectorSpaceClusterer(ClusterI):
         cluster = self.classify_vectorspace(vector)
         return self.cluster_name(cluster)
 
+    @abstractmethod
     def classify_vectorspace(self, vector):
         """
         Returns the index of the appropriate cluster for the vector.
         """
-        raise NotImplementedError()
 
     def likelihood(self, vector, label):
         if self._should_normalise:
@@ -92,7 +97,7 @@ class VectorSpaceClusterer(ClusterI):
         Returns the likelihood of the vector belonging to the cluster.
         """
         predicted = self.classify_vectorspace(vector)
-        return (1.0 if cluster == predicted else 0.0)
+        return 1.0 if cluster == predicted else 0.0
 
     def vector(self, vector):
         """
@@ -110,6 +115,7 @@ class VectorSpaceClusterer(ClusterI):
         """
         return vector / sqrt(numpy.dot(vector, vector))
 
+
 def euclidean_distance(u, v):
     """
     Returns the euclidean distance between vectors u and v. This is equivalent
@@ -118,13 +124,15 @@ def euclidean_distance(u, v):
     diff = u - v
     return sqrt(numpy.dot(diff, diff))
 
+
 def cosine_distance(u, v):
     """
-    Returns 1 minus the cosine of the angle between vectors v and u. This is equal to
-    1 - (u.v / |u||v|).
+    Returns 1 minus the cosine of the angle between vectors v and u. This is
+    equal to 1 - (u.v / |u||v|).
     """
     return 1 - (numpy.dot(u, v) / (sqrt(numpy.dot(u, u)) * sqrt(numpy.dot(v, v))))
 
+
 class _DendrogramNode(object):
     """ Tree node of a dendrogram. """
 
@@ -164,6 +172,9 @@ class _DendrogramNode(object):
             groups.append(node.leaves())
         return groups
 
+    def __lt__(self, comparator):
+        return cosine_distance(self._value, comparator._value) < 0
+
 
 @python_2_unicode_compatible
 class Dendrogram(object):
@@ -214,7 +225,8 @@ class Dendrogram(object):
     def show(self, leaf_labels=[]):
         """
         Print the dendrogram in ASCII art to standard out.
-        :param leaf_labels: an optional list of strings to use for labeling the leaves
+        :param leaf_labels: an optional list of strings to use for labeling the
+                            leaves
         :type leaf_labels: list
         """
 
@@ -235,18 +247,19 @@ class Dendrogram(object):
 
         # find the bottom row and the best cell width
         width = max(map(len, last_row)) + 1
-        lhalf = width / 2
-        rhalf = width - lhalf - 1
+        lhalf = width // 2
+        rhalf = int(width - lhalf - 1)
 
         # display functions
         def format(centre, left=' ', right=' '):
-            return '%s%s%s' % (lhalf*left, centre, right*rhalf)
+            return '%s%s%s' % (lhalf * left, centre, right * rhalf)
+
         def display(str):
             stdout.write(str)
 
         # for each merge, top down
         queue = [(root._value, root)]
-        verticals = [ format(' ') for leaf in leaves ]
+        verticals = [format(' ') for leaf in leaves]
         while queue:
             priority, node = queue.pop()
             child_left_leaf = list(map(lambda c: c.leaves(False)[0], node._children))
@@ -256,9 +269,12 @@ class Dendrogram(object):
                 max_idx = max(indices)
             for i in range(len(leaves)):
                 if leaves[i] in child_left_leaf:
-                    if i == min_idx:    display(format(JOIN, ' ', HLINK))
-                    elif i == max_idx:  display(format(JOIN, HLINK, ' '))
-                    else:               display(format(JOIN, HLINK, HLINK))
+                    if i == min_idx:
+                        display(format(JOIN, ' ', HLINK))
+                    elif i == max_idx:
+                        display(format(JOIN, HLINK, ' '))
+                    else:
+                        display(format(JOIN, HLINK, HLINK))
                     verticals[i] = format(VLINK)
                 elif min_idx <= i <= max_idx:
                     display(format(HLINK, HLINK, HLINK))
@@ -285,5 +301,3 @@ class Dendrogram(object):
             root = self._items[0]
         leaves = root.leaves(False)
         return '<Dendrogram with %d leaves>' % len(leaves)
-
-
