@@ -1,6 +1,6 @@
 # Natural Language Toolkit: Corpus Readers
 #
-# Copyright (C) 2001-2016 NLTK Project
+# Copyright (C) 2001-2019 NLTK Project
 # Author: Steven Bird <stevenbird1@gmail.com>
 #         Edward Loper <edloper@gmail.com>
 # URL: <http://nltk.org/>
@@ -103,43 +103,81 @@ from nltk.corpus.reader.pros_cons import *
 from nltk.corpus.reader.categorized_sents import *
 from nltk.corpus.reader.comparative_sents import *
 from nltk.corpus.reader.panlex_lite import *
+from nltk.corpus.reader.panlex_swadesh import *
 
 # Make sure that nltk.corpus.reader.bracket_parse gives the module, not
 # the function bracket_parse() defined in nltk.tree:
 from nltk.corpus.reader import bracket_parse
 
 __all__ = [
-    'CorpusReader', 'CategorizedCorpusReader',
-    'PlaintextCorpusReader', 'find_corpus_fileids',
-    'TaggedCorpusReader', 'CMUDictCorpusReader',
-    'ConllChunkCorpusReader', 'WordListCorpusReader',
-    'PPAttachmentCorpusReader', 'SensevalCorpusReader',
-    'IEERCorpusReader', 'ChunkedCorpusReader',
-    'SinicaTreebankCorpusReader', 'BracketParseCorpusReader',
-    'IndianCorpusReader', 'ToolboxCorpusReader',
-    'TimitCorpusReader', 'YCOECorpusReader',
-    'MacMorphoCorpusReader', 'SyntaxCorpusReader',
-    'AlpinoCorpusReader', 'RTECorpusReader',
-    'StringCategoryCorpusReader','EuroparlCorpusReader',
+    'CorpusReader',
+    'CategorizedCorpusReader',
+    'PlaintextCorpusReader',
+    'find_corpus_fileids',
+    'TaggedCorpusReader',
+    'CMUDictCorpusReader',
+    'ConllChunkCorpusReader',
+    'WordListCorpusReader',
+    'PPAttachmentCorpusReader',
+    'SensevalCorpusReader',
+    'IEERCorpusReader',
+    'ChunkedCorpusReader',
+    'SinicaTreebankCorpusReader',
+    'BracketParseCorpusReader',
+    'IndianCorpusReader',
+    'ToolboxCorpusReader',
+    'TimitCorpusReader',
+    'YCOECorpusReader',
+    'MacMorphoCorpusReader',
+    'SyntaxCorpusReader',
+    'AlpinoCorpusReader',
+    'RTECorpusReader',
+    'StringCategoryCorpusReader',
+    'EuroparlCorpusReader',
     'CategorizedBracketParseCorpusReader',
     'CategorizedTaggedCorpusReader',
     'CategorizedPlaintextCorpusReader',
     'PortugueseCategorizedPlaintextCorpusReader',
     'tagged_treebank_para_block_reader',
-    'PropbankCorpusReader', 'VerbnetCorpusReader',
-    'BNCCorpusReader', 'ConllCorpusReader',
-    'XMLCorpusReader', 'NPSChatCorpusReader',
-    'SwadeshCorpusReader', 'WordNetCorpusReader',
-    'WordNetICCorpusReader', 'SwitchboardCorpusReader',
-    'DependencyCorpusReader', 'NombankCorpusReader',
-    'IPIPANCorpusReader', 'Pl196xCorpusReader',
-    'TEICorpusView', 'KNBCorpusReader', 'ChasenCorpusReader',
-    'CHILDESCorpusReader', 'AlignedCorpusReader',
-    'TimitTaggedCorpusReader', 'LinThesaurusCorpusReader',
-    'SemcorCorpusReader', 'FramenetCorpusReader', 'UdhrCorpusReader',
-    'BNCCorpusReader', 'SentiWordNetCorpusReader', 'SentiSynset',
-    'TwitterCorpusReader', 'NKJPCorpusReader', 'CrubadanCorpusReader',
-    'MTECorpusReader', 'ReviewsCorpusReader', 'OpinionLexiconCorpusReader',
-    'ProsConsCorpusReader', 'CategorizedSentencesCorpusReader',
-    'ComparativeSentencesCorpusReader', 'PanLexLiteCorpusReader'
+    'PropbankCorpusReader',
+    'VerbnetCorpusReader',
+    'BNCCorpusReader',
+    'ConllCorpusReader',
+    'XMLCorpusReader',
+    'NPSChatCorpusReader',
+    'SwadeshCorpusReader',
+    'WordNetCorpusReader',
+    'WordNetICCorpusReader',
+    'SwitchboardCorpusReader',
+    'DependencyCorpusReader',
+    'NombankCorpusReader',
+    'IPIPANCorpusReader',
+    'Pl196xCorpusReader',
+    'TEICorpusView',
+    'KNBCorpusReader',
+    'ChasenCorpusReader',
+    'CHILDESCorpusReader',
+    'AlignedCorpusReader',
+    'TimitTaggedCorpusReader',
+    'LinThesaurusCorpusReader',
+    'SemcorCorpusReader',
+    'FramenetCorpusReader',
+    'UdhrCorpusReader',
+    'BNCCorpusReader',
+    'SentiWordNetCorpusReader',
+    'SentiSynset',
+    'TwitterCorpusReader',
+    'NKJPCorpusReader',
+    'CrubadanCorpusReader',
+    'MTECorpusReader',
+    'ReviewsCorpusReader',
+    'OpinionLexiconCorpusReader',
+    'ProsConsCorpusReader',
+    'CategorizedSentencesCorpusReader',
+    'ComparativeSentencesCorpusReader',
+    'PanLexLiteCorpusReader',
+    'NonbreakingPrefixesCorpusReader',
+    'UnicharsCorpusReader',
+    'MWAPPDBCorpusReader',
+    'PanlexSwadeshCorpusReader',
 ]
