@@ -2,17 +2,19 @@
 A reader for corpora whose documents are in MTE format.
 """
 import os
+import re
 from functools import reduce
-from nltk import compat
+
+from six import string_types
+
 from nltk.corpus.reader import concat, TaggedCorpusReader
 from nltk.corpus.reader.xmldocs import XMLCorpusView
 
-import xml.etree.ElementTree as etree
-import re
 
 def xpath(root, path, ns):
     return root.findall(path, ns)
 
+
 class MTECorpusView(XMLCorpusView):
     """
     Class for lazy viewing the MTE Corpus.
@@ -22,7 +24,13 @@ class MTECorpusView(XMLCorpusView):
         XMLCorpusView.__init__(self, fileid, tagspec, elt_handler)
 
     def read_block(self, stream, tagspec=None, elt_handler=None):
-        return list(filter(lambda x: x is not None, XMLCorpusView.read_block(self, stream, tagspec, elt_handler)))
+        return list(
+            filter(
+                lambda x: x is not None,
+                XMLCorpusView.read_block(self, stream, tagspec, elt_handler),
+            )
+        )
+
 
 class MTEFileReader:
     """
@@ -30,104 +38,137 @@ class MTEFileReader:
     parses the xml files and does some tag-filtering depending on the
     given method parameters.
     """
-    ns = {'tei': 'http://www.tei-c.org/ns/1.0',
-          'xml': 'http://www.w3.org/XML/1998/namespace'}
+
+    ns = {
+        'tei': 'http://www.tei-c.org/ns/1.0',
+        'xml': 'http://www.w3.org/XML/1998/namespace',
+    }
     tag_ns = '{http://www.tei-c.org/ns/1.0}'
     xml_ns = '{http://www.w3.org/XML/1998/namespace}'
     word_path = "TEI/text/body/div/div/p/s/(w|c)"
     sent_path = "TEI/text/body/div/div/p/s"
     para_path = "TEI/text/body/div/div/p"
 
-
     def __init__(self, file_path):
         self.__file_path = file_path
 
     @classmethod
-    def _word_elt(self, elt, context):
+    def _word_elt(cls, elt, context):
         return elt.text
 
     @classmethod
-    def _sent_elt(self, elt, context):
-        return [self._word_elt(w, None) for w in xpath(elt, '*', self.ns)]
+    def _sent_elt(cls, elt, context):
+        return [cls._word_elt(w, None) for w in xpath(elt, '*', cls.ns)]
 
     @classmethod
-    def _para_elt(self, elt, context):
-        return [self._sent_elt(s, None) for s in xpath(elt, '*', self.ns)]
+    def _para_elt(cls, elt, context):
+        return [cls._sent_elt(s, None) for s in xpath(elt, '*', cls.ns)]
 
     @classmethod
-    def _tagged_word_elt(self, elt, context):
-        if ('ana' not in elt.attrib):
+    def _tagged_word_elt(cls, elt, context):
+        if 'ana' not in elt.attrib:
             return (elt.text, '')
 
-        if self.__tags == "" and self.__tagset == "msd":
+        if cls.__tags == "" and cls.__tagset == "msd":
             return (elt.text, elt.attrib['ana'])
-        elif self.__tags == "" and self.__tagset == "universal":
+        elif cls.__tags == "" and cls.__tagset == "universal":
             return (elt.text, MTETagConverter.msd_to_universal(elt.attrib['ana']))
         else:
-            tags = re.compile('^' + re.sub("-", ".", self.__tags) + '.*$')
-            if (tags.match(elt.attrib['ana'])):
-                if self.__tagset == "msd":
+            tags = re.compile('^' + re.sub("-", ".", cls.__tags) + '.*$')
+            if tags.match(elt.attrib['ana']):
+                if cls.__tagset == "msd":
                     return (elt.text, elt.attrib['ana'])
                 else:
-                    return (elt.text, MTETagConverter.msd_to_universal(elt.attrib['ana']))
+                    return (
+                        elt.text,
+                        MTETagConverter.msd_to_universal(elt.attrib['ana']),
+                    )
             else:
                 return None
 
     @classmethod
-    def _tagged_sent_elt(self, elt, context):
-        return list(filter(lambda x: x is not None, [self._tagged_word_elt(w, None) for w in xpath(elt, '*', self.ns)]))
+    def _tagged_sent_elt(cls, elt, context):
+        return list(
+            filter(
+                lambda x: x is not None,
+                [cls._tagged_word_elt(w, None) for w in xpath(elt, '*', cls.ns)],
+            )
+        )
 
     @classmethod
-    def _tagged_para_elt(self, elt, context):
-        return list(filter(lambda x: x is not None, [self._tagged_sent_elt(s, None) for s in xpath(elt, '*', self.ns)]))
+    def _tagged_para_elt(cls, elt, context):
+        return list(
+            filter(
+                lambda x: x is not None,
+                [cls._tagged_sent_elt(s, None) for s in xpath(elt, '*', cls.ns)],
+            )
+        )
 
     @classmethod
-    def _lemma_word_elt(self, elt, context):
-        if ('lemma' not in elt.attrib):
+    def _lemma_word_elt(cls, elt, context):
+        if 'lemma' not in elt.attrib:
             return (elt.text, '')
         else:
             return (elt.text, elt.attrib['lemma'])
 
     @classmethod
-    def _lemma_sent_elt(self, elt, context):
-        return [self._lemma_word_elt(w, None) for w in xpath(elt, '*', self.ns)]
+    def _lemma_sent_elt(cls, elt, context):
+        return [cls._lemma_word_elt(w, None) for w in xpath(elt, '*', cls.ns)]
 
     @classmethod
-    def _lemma_para_elt(self, elt, context):
-        return [self._lemma_sent_elt(s, None) for s in xpath(elt, '*', self.ns)]
+    def _lemma_para_elt(cls, elt, context):
+        return [cls._lemma_sent_elt(s, None) for s in xpath(elt, '*', cls.ns)]
 
     def words(self):
-        return MTECorpusView(self.__file_path, MTEFileReader.word_path, MTEFileReader._word_elt)
+        return MTECorpusView(
+            self.__file_path, MTEFileReader.word_path, MTEFileReader._word_elt
+        )
 
     def sents(self):
-        return MTECorpusView(self.__file_path, MTEFileReader.sent_path, MTEFileReader._sent_elt)
+        return MTECorpusView(
+            self.__file_path, MTEFileReader.sent_path, MTEFileReader._sent_elt
+        )
 
     def paras(self):
-        return MTECorpusView(self.__file_path, MTEFileReader.para_path, MTEFileReader._para_elt)
+        return MTECorpusView(
+            self.__file_path, MTEFileReader.para_path, MTEFileReader._para_elt
+        )
 
     def lemma_words(self):
-        return MTECorpusView(self.__file_path, MTEFileReader.word_path, MTEFileReader._lemma_word_elt)
+        return MTECorpusView(
+            self.__file_path, MTEFileReader.word_path, MTEFileReader._lemma_word_elt
+        )
 
     def tagged_words(self, tagset, tags):
         MTEFileReader.__tagset = tagset
         MTEFileReader.__tags = tags
-        return MTECorpusView(self.__file_path, MTEFileReader.word_path, MTEFileReader._tagged_word_elt)
+        return MTECorpusView(
+            self.__file_path, MTEFileReader.word_path, MTEFileReader._tagged_word_elt
+        )
 
     def lemma_sents(self):
-        return MTECorpusView(self.__file_path, MTEFileReader.sent_path, MTEFileReader._lemma_sent_elt)
+        return MTECorpusView(
+            self.__file_path, MTEFileReader.sent_path, MTEFileReader._lemma_sent_elt
+        )
 
     def tagged_sents(self, tagset, tags):
         MTEFileReader.__tagset = tagset
         MTEFileReader.__tags = tags
-        return MTECorpusView(self.__file_path, MTEFileReader.sent_path, MTEFileReader._tagged_sent_elt)
+        return MTECorpusView(
+            self.__file_path, MTEFileReader.sent_path, MTEFileReader._tagged_sent_elt
+        )
 
     def lemma_paras(self):
-        return MTECorpusView(self.__file_path, MTEFileReader.para_path, MTEFileReader._lemma_para_elt)
+        return MTECorpusView(
+            self.__file_path, MTEFileReader.para_path, MTEFileReader._lemma_para_elt
+        )
 
     def tagged_paras(self, tagset, tags):
         MTEFileReader.__tagset = tagset
         MTEFileReader.__tags = tags
-        return MTECorpusView(self.__file_path, MTEFileReader.para_path, MTEFileReader._tagged_para_elt)
+        return MTECorpusView(
+            self.__file_path, MTEFileReader.para_path, MTEFileReader._tagged_para_elt
+        )
 
 
 class MTETagConverter:
@@ -137,9 +178,19 @@ class MTETagConverter:
     """
 
     mapping_msd_universal = {
-        'A': 'ADJ', 'S': 'ADP', 'R': 'ADV', 'C': 'CONJ',
-        'D': 'DET', 'N': 'NOUN', 'M': 'NUM', 'Q': 'PRT',
-        'P': 'PRON', 'V': 'VERB', '.': '.', '-': 'X'}
+        'A': 'ADJ',
+        'S': 'ADP',
+        'R': 'ADV',
+        'C': 'CONJ',
+        'D': 'DET',
+        'N': 'NOUN',
+        'M': 'NUM',
+        'Q': 'PRT',
+        'P': 'PRON',
+        'V': 'VERB',
+        '.': '.',
+        '-': 'X',
+    }
 
     @staticmethod
     def msd_to_universal(tag):
@@ -156,6 +207,7 @@ class MTETagConverter:
 
         return MTETagConverter.mapping_msd_universal[indicator]
 
+
 class MTECorpusReader(TaggedCorpusReader):
     """
     Reader for corpora following the TEI-p5 xml scheme, such as MULTEXT-East.
@@ -178,12 +230,14 @@ class MTECorpusReader(TaggedCorpusReader):
         TaggedCorpusReader.__init__(self, root, fileids, encoding)
 
     def __fileids(self, fileids):
-        if fileids is None: fileids = self._fileids
-        elif isinstance(fileids, compat.string_types): fileids = [fileids]
+        if fileids is None:
+            fileids = self._fileids
+        elif isinstance(fileids, string_types):
+            fileids = [fileids]
         # filter wrong userinput
-        fileids = filter(lambda x : x in self._fileids, fileids)
+        fileids = filter(lambda x: x in self._fileids, fileids)
         # filter multext-east sourcefiles that are not compatible to the teip5 specification
-        fileids = filter(lambda x : x not in ["oana-bg.xml", "oana-mk.xml"], fileids)
+        fileids = filter(lambda x: x not in ["oana-bg.xml", "oana-mk.xml"], fileids)
         if not fileids:
             print("No valid multext-east file specified")
         return fileids
@@ -210,7 +264,12 @@ class MTECorpusReader(TaggedCorpusReader):
         :return: the given file(s) as a list of words and punctuation symbols.
         :rtype: list(str)
         """
-        return  concat([MTEFileReader(os.path.join(self._root, f)).words() for f in self.__fileids(fileids)])
+        return concat(
+            [
+                MTEFileReader(os.path.join(self._root, f)).words()
+                for f in self.__fileids(fileids)
+            ]
+        )
 
     def sents(self, fileids=None):
         """
@@ -219,7 +278,12 @@ class MTECorpusReader(TaggedCorpusReader):
                  each encoded as a list of word strings
         :rtype: list(list(str))
         """
-        return  concat([MTEFileReader(os.path.join(self._root, f)).sents() for f in self.__fileids(fileids)])
+        return concat(
+            [
+                MTEFileReader(os.path.join(self._root, f)).sents()
+                for f in self.__fileids(fileids)
+            ]
+        )
 
     def paras(self, fileids=None):
         """
@@ -228,7 +292,12 @@ class MTECorpusReader(TaggedCorpusReader):
                  of sentences, which are in turn encoded as lists of word string
         :rtype: list(list(list(str)))
         """
-        return  concat([MTEFileReader(os.path.join(self._root, f)).paras() for f in self.__fileids(fileids)])
+        return concat(
+            [
+                MTEFileReader(os.path.join(self._root, f)).paras()
+                for f in self.__fileids(fileids)
+            ]
+        )
 
     def lemma_words(self, fileids=None):
         """
@@ -237,7 +306,12 @@ class MTECorpusReader(TaggedCorpusReader):
                  and punctuation symbols, encoded as tuples (word, lemma)
         :rtype: list(tuple(str,str))
         """
-        return  concat([MTEFileReader(os.path.join(self._root, f)).lemma_words() for f in self.__fileids(fileids)])
+        return concat(
+            [
+                MTEFileReader(os.path.join(self._root, f)).lemma_words()
+                for f in self.__fileids(fileids)
+            ]
+        )
 
     def tagged_words(self, fileids=None, tagset="msd", tags=""):
         """
@@ -251,7 +325,14 @@ class MTECorpusReader(TaggedCorpusReader):
         :rtype: list(tuple(str, str))
         """
         if tagset == "universal" or tagset == "msd":
-            return concat([MTEFileReader(os.path.join(self._root, f)).tagged_words(tagset, tags) for f in self.__fileids(fileids)])
+            return concat(
+                [
+                    MTEFileReader(os.path.join(self._root, f)).tagged_words(
+                        tagset, tags
+                    )
+                    for f in self.__fileids(fileids)
+                ]
+            )
         else:
             print("Unknown tagset specified.")
 
@@ -263,8 +344,12 @@ class MTECorpusReader(TaggedCorpusReader):
                  lemma (word, lemma)
         :rtype: list(list(tuple(str, str)))
         """
-        return  concat([MTEFileReader(os.path.join(self._root, f)).lemma_sents() for f in self.__fileids(fileids)])
-
+        return concat(
+            [
+                MTEFileReader(os.path.join(self._root, f)).lemma_sents()
+                for f in self.__fileids(fileids)
+            ]
+        )
 
     def tagged_sents(self, fileids=None, tagset="msd", tags=""):
         """
@@ -278,7 +363,14 @@ class MTECorpusReader(TaggedCorpusReader):
         :rtype: list(list(tuple(str, str)))
         """
         if tagset == "universal" or tagset == "msd":
-            return concat([MTEFileReader(os.path.join(self._root, f)).tagged_sents(tagset, tags) for f in self.__fileids(fileids)])
+            return concat(
+                [
+                    MTEFileReader(os.path.join(self._root, f)).tagged_sents(
+                        tagset, tags
+                    )
+                    for f in self.__fileids(fileids)
+                ]
+            )
         else:
             print("Unknown tagset specified.")
 
@@ -290,7 +382,12 @@ class MTECorpusReader(TaggedCorpusReader):
                  tuples of the word and the corresponding lemma (word, lemma)
         :rtype: list(List(List(tuple(str, str))))
         """
-        return concat([MTEFileReader(os.path.join(self._root, f)).lemma_paras() for f in self.__fileids(fileids)])
+        return concat(
+            [
+                MTEFileReader(os.path.join(self._root, f)).lemma_paras()
+                for f in self.__fileids(fileids)
+            ]
+        )
 
     def tagged_paras(self, fileids=None, tagset="msd", tags=""):
         """
@@ -305,6 +402,13 @@ class MTECorpusReader(TaggedCorpusReader):
         :rtype: list(list(list(tuple(str, str))))
         """
         if tagset == "universal" or tagset == "msd":
-            return concat([MTEFileReader(os.path.join(self._root, f)).tagged_paras(tagset, tags) for f in self.__fileids(fileids)])
+            return concat(
+                [
+                    MTEFileReader(os.path.join(self._root, f)).tagged_paras(
+                        tagset, tags
+                    )
+                    for f in self.__fileids(fileids)
+                ]
+            )
         else:
             print("Unknown tagset specified.")
