@@ -1,6 +1,6 @@
 # Natural Language Toolkit: CFG visualization
 #
-# Copyright (C) 2001-2016 NLTK Project
+# Copyright (C) 2001-2019 NLTK Project
 # Author: Edward Loper <edloper@gmail.com>
 # URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
@@ -46,34 +46,47 @@ Visualization tools for CFGs.
 #   - disconnect top & bottom -- right click
 #     - if connected to top & bottom, then disconnect
 
-
-
-import nltk.compat
 import re
 
-from tkinter import (Button, Canvas, Entry, Frame, IntVar, Label,
-                     Scrollbar, Text, Tk, Toplevel)
+from six import string_types
+from six.moves.tkinter import (
+    Button,
+    Canvas,
+    Entry,
+    Frame,
+    IntVar,
+    Label,
+    Scrollbar,
+    Text,
+    Tk,
+    Toplevel,
+)
 
-from nltk.grammar import (CFG, _read_cfg_production,
-                          Nonterminal, nonterminals)
+from nltk.grammar import CFG, _read_cfg_production, Nonterminal, nonterminals
 from nltk.tree import Tree
 from nltk.draw.tree import TreeSegmentWidget, tree_to_treesegment
-from nltk.draw.util import (CanvasFrame, ColorizedList, ShowText,
-                            SymbolWidget, TextWidget)
-from nltk import compat
+from nltk.draw.util import (
+    CanvasFrame,
+    ColorizedList,
+    ShowText,
+    SymbolWidget,
+    TextWidget,
+)
 
 ######################################################################
 # Production List
 ######################################################################
 
+
 class ProductionList(ColorizedList):
     ARROW = SymbolWidget.SYMBOLS['rightarrow']
 
     def _init_colortags(self, textwidget, options):
         textwidget.tag_config('terminal', foreground='#006000')
         textwidget.tag_config('arrow', font='symbol', underline='0')
-        textwidget.tag_config('nonterminal', foreground='blue',
-                              font=('helvetica', -12, 'bold'))
+        textwidget.tag_config(
+            'nonterminal', foreground='blue', font=('helvetica', -12, 'bold')
+        )
 
     def _item_repr(self, item):
         contents = []
@@ -86,6 +99,7 @@ class ProductionList(ColorizedList):
                 contents.append((' %r' % elt, 'terminal'))
         return contents
 
+
 ######################################################################
 # CFG Editor
 ######################################################################
@@ -133,6 +147,7 @@ the CFG:
 
 """
 
+
 class CFGEditor(object):
     """
     A dialog window for creating and editing context free grammars.
@@ -143,21 +158,28 @@ class CFGEditor(object):
     - All terminals must be strings consisting of word characters
       and space characters.
     """
+
     # Regular expressions used by _analyze_line.  Precompile them, so
     # we can process the text faster.
     ARROW = SymbolWidget.SYMBOLS['rightarrow']
-    _LHS_RE = re.compile(r"(^\s*\w+\s*)(->|("+ARROW+"))")
-    _ARROW_RE = re.compile("\s*(->|("+ARROW+"))\s*")
-    _PRODUCTION_RE = re.compile(r"(^\s*\w+\s*)" +              # LHS
-                                "(->|("+ARROW+"))\s*" +        # arrow
-                                r"((\w+|'[\w ]*'|\"[\w ]*\"|\|)\s*)*$") # RHS
-    _TOKEN_RE = re.compile("\\w+|->|'[\\w ]+'|\"[\\w ]+\"|("+ARROW+")")
+    _LHS_RE = re.compile(r"(^\s*\w+\s*)(->|(" + ARROW + "))")
+    _ARROW_RE = re.compile("\s*(->|(" + ARROW + "))\s*")
+    _PRODUCTION_RE = re.compile(
+        r"(^\s*\w+\s*)"
+        + "(->|("  # LHS
+        + ARROW
+        + "))\s*"
+        + r"((\w+|'[\w ]*'|\"[\w ]*\"|\|)\s*)*$"  # arrow
+    )  # RHS
+    _TOKEN_RE = re.compile("\\w+|->|'[\\w ]+'|\"[\\w ]+\"|(" + ARROW + ")")
     _BOLD = ('helvetica', -12, 'bold')
 
     def __init__(self, parent, cfg=None, set_cfg_callback=None):
         self._parent = parent
-        if cfg is not None: self._cfg = cfg
-        else: self._cfg = CFG(Nonterminal('S'), [])
+        if cfg is not None:
+            self._cfg = cfg
+        else:
+            self._cfg = CFG(Nonterminal('S'), [])
         self._set_cfg_callback = set_cfg_callback
 
         self._highlight_matching_nonterminals = 1
@@ -185,26 +207,31 @@ class CFGEditor(object):
 
     def _init_buttons(self):
         frame = self._buttonframe = Frame(self._top)
-        Button(frame, text='Ok', command=self._ok,
-               underline=0, takefocus=0).pack(side='left')
-        Button(frame, text='Apply', command=self._apply,
-               underline=0, takefocus=0).pack(side='left')
-        Button(frame, text='Reset', command=self._reset,
-               underline=0, takefocus=0,).pack(side='left')
-        Button(frame, text='Cancel', command=self._cancel,
-               underline=0, takefocus=0).pack(side='left')
-        Button(frame, text='Help', command=self._help,
-               underline=0, takefocus=0).pack(side='right')
+        Button(frame, text='Ok', command=self._ok, underline=0, takefocus=0).pack(
+            side='left'
+        )
+        Button(frame, text='Apply', command=self._apply, underline=0, takefocus=0).pack(
+            side='left'
+        )
+        Button(frame, text='Reset', command=self._reset, underline=0, takefocus=0).pack(
+            side='left'
+        )
+        Button(
+            frame, text='Cancel', command=self._cancel, underline=0, takefocus=0
+        ).pack(side='left')
+        Button(frame, text='Help', command=self._help, underline=0, takefocus=0).pack(
+            side='right'
+        )
 
     def _init_bindings(self):
         self._top.title('CFG Editor')
         self._top.bind('<Control-q>', self._cancel)
         self._top.bind('<Alt-q>', self._cancel)
         self._top.bind('<Control-d>', self._cancel)
-        #self._top.bind('<Control-x>', self._cancel)
+        # self._top.bind('<Control-x>', self._cancel)
         self._top.bind('<Alt-x>', self._cancel)
         self._top.bind('<Escape>', self._cancel)
-        #self._top.bind('<Control-c>', self._cancel)
+        # self._top.bind('<Control-c>', self._cancel)
         self._top.bind('<Alt-c>', self._cancel)
 
         self._top.bind('<Control-o>', self._ok)
@@ -221,11 +248,11 @@ class CFGEditor(object):
         self._prodframe = Frame(self._top)
 
         # Create the basic Text widget & scrollbar.
-        self._textwidget = Text(self._prodframe, background='#e0e0e0',
-                                exportselection=1)
-        self._textscroll = Scrollbar(self._prodframe, takefocus=0,
-                                     orient='vertical')
-        self._textwidget.config(yscrollcommand = self._textscroll.set)
+        self._textwidget = Text(
+            self._prodframe, background='#e0e0e0', exportselection=1
+        )
+        self._textscroll = Scrollbar(self._prodframe, takefocus=0, orient='vertical')
+        self._textwidget.config(yscrollcommand=self._textscroll.set)
         self._textscroll.config(command=self._textwidget.yview)
         self._textscroll.pack(side='right', fill='y')
         self._textwidget.pack(expand=1, fill='both', side='left')
@@ -251,16 +278,19 @@ class CFGEditor(object):
         # Tab cycles focus. (why doesn't this work??)
         def cycle(e, textwidget=self._textwidget):
             textwidget.tk_focusNext().focus()
+
         self._textwidget.bind('<Tab>', cycle)
 
-        prod_tuples = [(p.lhs(),[p.rhs()]) for p in self._cfg.productions()]
-        for i in range(len(prod_tuples)-1,0,-1):
-            if (prod_tuples[i][0] == prod_tuples[i-1][0]):
-                if () in prod_tuples[i][1]: continue
-                if () in prod_tuples[i-1][1]: continue
-                print(prod_tuples[i-1][1])
+        prod_tuples = [(p.lhs(), [p.rhs()]) for p in self._cfg.productions()]
+        for i in range(len(prod_tuples) - 1, 0, -1):
+            if prod_tuples[i][0] == prod_tuples[i - 1][0]:
+                if () in prod_tuples[i][1]:
+                    continue
+                if () in prod_tuples[i - 1][1]:
+                    continue
+                print(prod_tuples[i - 1][1])
                 print(prod_tuples[i][1])
-                prod_tuples[i-1][1].extend(prod_tuples[i][1])
+                prod_tuples[i - 1][1].extend(prod_tuples[i][1])
                 del prod_tuples[i]
 
         for lhs, rhss in prod_tuples:
@@ -268,46 +298,48 @@ class CFGEditor(object):
             s = '%s ->' % lhs
             for rhs in rhss:
                 for elt in rhs:
-                    if isinstance(elt, Nonterminal): s += ' %s' % elt
-                    else: s += ' %r' % elt
+                    if isinstance(elt, Nonterminal):
+                        s += ' %s' % elt
+                    else:
+                        s += ' %r' % elt
                 s += ' |'
             s = s[:-2] + '\n'
             self._textwidget.insert('end', s)
 
         self._analyze()
 
-#         # Add the producitons to the text widget, and colorize them.
-#         prod_by_lhs = {}
-#         for prod in self._cfg.productions():
-#             if len(prod.rhs()) > 0:
-#                 prod_by_lhs.setdefault(prod.lhs(),[]).append(prod)
-#         for (lhs, prods) in prod_by_lhs.items():
-#             self._textwidget.insert('end', '%s ->' % lhs)
-#             self._textwidget.insert('end', self._rhs(prods[0]))
-#             for prod in prods[1:]:
-#                 print '\t|'+self._rhs(prod),
-#                 self._textwidget.insert('end', '\t|'+self._rhs(prod))
-#             print
-#             self._textwidget.insert('end', '\n')
-#         for prod in self._cfg.productions():
-#             if len(prod.rhs()) == 0:
-#                 self._textwidget.insert('end', '%s' % prod)
-#         self._analyze()
+    #         # Add the producitons to the text widget, and colorize them.
+    #         prod_by_lhs = {}
+    #         for prod in self._cfg.productions():
+    #             if len(prod.rhs()) > 0:
+    #                 prod_by_lhs.setdefault(prod.lhs(),[]).append(prod)
+    #         for (lhs, prods) in prod_by_lhs.items():
+    #             self._textwidget.insert('end', '%s ->' % lhs)
+    #             self._textwidget.insert('end', self._rhs(prods[0]))
+    #             for prod in prods[1:]:
+    #                 print '\t|'+self._rhs(prod),
+    #                 self._textwidget.insert('end', '\t|'+self._rhs(prod))
+    #             print
+    #             self._textwidget.insert('end', '\n')
+    #         for prod in self._cfg.productions():
+    #             if len(prod.rhs()) == 0:
+    #                 self._textwidget.insert('end', '%s' % prod)
+    #         self._analyze()
 
-#     def _rhs(self, prod):
-#         s = ''
-#         for elt in prod.rhs():
-#             if isinstance(elt, Nonterminal): s += ' %s' % elt.symbol()
-#             else: s += ' %r' % elt
-#         return s
+    #     def _rhs(self, prod):
+    #         s = ''
+    #         for elt in prod.rhs():
+    #             if isinstance(elt, Nonterminal): s += ' %s' % elt.symbol()
+    #             else: s += ' %r' % elt
+    #         return s
 
     def _clear_tags(self, linenum):
         """
         Remove all tags (except ``arrow`` and ``sel``) from the given
         line of the text widget used for editing the productions.
         """
-        start = '%d.0'%linenum
-        end = '%d.end'%linenum
+        start = '%d.0' % linenum
+        end = '%d.end' % linenum
         for tag in self._textwidget.tag_names():
             if tag not in ('arrow', 'sel'):
                 self._textwidget.tag_remove(tag, start, end)
@@ -333,17 +365,18 @@ class CFGEditor(object):
         arrow = '1.0'
         while True:
             arrow = self._textwidget.search('->', arrow, 'end+1char')
-            if arrow == '': break
-            self._textwidget.delete(arrow, arrow+'+2char')
+            if arrow == '':
+                break
+            self._textwidget.delete(arrow, arrow + '+2char')
             self._textwidget.insert(arrow, self.ARROW, 'arrow')
             self._textwidget.insert(arrow, '\t')
 
         arrow = '1.0'
         while True:
-            arrow = self._textwidget.search(self.ARROW, arrow+'+1char',
-                                            'end+1char')
-            if arrow == '': break
-            self._textwidget.tag_add('arrow', arrow, arrow+'+1char')
+            arrow = self._textwidget.search(self.ARROW, arrow + '+1char', 'end+1char')
+            if arrow == '':
+                break
+            self._textwidget.tag_add('arrow', arrow, arrow + '+1char')
 
     def _analyze_token(self, match, linenum):
         """
@@ -353,13 +386,15 @@ class CFGEditor(object):
         the line).
         """
         # What type of token is it?
-        if match.group()[0] in "'\"": tag = 'terminal'
-        elif match.group() in ('->', self.ARROW): tag = 'arrow'
+        if match.group()[0] in "'\"":
+            tag = 'terminal'
+        elif match.group() in ('->', self.ARROW):
+            tag = 'arrow'
         else:
             # If it's a nonterminal, then set up new bindings, so we
             # can highlight all instances of that nonterminal when we
             # put the mouse over it.
-            tag = 'nonterminal_'+match.group()
+            tag = 'nonterminal_' + match.group()
             if tag not in self._textwidget.tag_names():
                 self._init_nonterminal_tag(tag)
 
@@ -368,14 +403,16 @@ class CFGEditor(object):
         self._textwidget.tag_add(tag, start, end)
 
     def _init_nonterminal_tag(self, tag, foreground='blue'):
-        self._textwidget.tag_config(tag, foreground=foreground,
-                                    font=CFGEditor._BOLD)
+        self._textwidget.tag_config(tag, foreground=foreground, font=CFGEditor._BOLD)
         if not self._highlight_matching_nonterminals:
             return
+
         def enter(e, textwidget=self._textwidget, tag=tag):
             textwidget.tag_config(tag, background='#80ff80')
+
         def leave(e, textwidget=self._textwidget, tag=tag):
             textwidget.tag_config(tag, background='')
+
         self._textwidget.tag_bind(tag, '<Enter>', enter)
         self._textwidget.tag_bind(tag, '<Leave>', leave)
 
@@ -387,7 +424,7 @@ class CFGEditor(object):
         self._clear_tags(linenum)
 
         # Get the line line's text string.
-        line = self._textwidget.get(repr(linenum)+'.0', repr(linenum)+'.end')
+        line = self._textwidget.get(repr(linenum) + '.0', repr(linenum) + '.end')
 
         # If it's a valid production, then colorize each token.
         if CFGEditor._PRODUCTION_RE.match(line):
@@ -396,6 +433,7 @@ class CFGEditor(object):
             def analyze_token(match, self=self, linenum=linenum):
                 self._analyze_token(match, linenum)
                 return ''
+
             CFGEditor._TOKEN_RE.sub(analyze_token, line)
         elif line.strip() != '':
             # It's invalid; show the user where the error is.
@@ -431,7 +469,7 @@ class CFGEditor(object):
         """
         self._replace_arrows()
         numlines = int(self._textwidget.index('end').split('.')[0])
-        for linenum in range(1, numlines+1):  # line numbers start at 1.
+        for linenum in range(1, numlines + 1):  # line numbers start at 1.
             self._analyze_line(linenum)
 
     def _parse_productions(self):
@@ -450,28 +488,30 @@ class CFGEditor(object):
         # Convert each line to a CFG production
         for line in lines:
             line = line.strip()
-            if line=='': continue
+            if line == '':
+                continue
             productions += _read_cfg_production(line)
-            #if line.strip() == '': continue
-            #if not CFGEditor._PRODUCTION_RE.match(line):
+            # if line.strip() == '': continue
+            # if not CFGEditor._PRODUCTION_RE.match(line):
             #    raise ValueError('Bad production string %r' % line)
             #
-            #(lhs_str, rhs_str) = line.split('->')
-            #lhs = Nonterminal(lhs_str.strip())
-            #rhs = []
-            #def parse_token(match, rhs=rhs):
+            # (lhs_str, rhs_str) = line.split('->')
+            # lhs = Nonterminal(lhs_str.strip())
+            # rhs = []
+            # def parse_token(match, rhs=rhs):
             #    token = match.group()
             #    if token[0] in "'\"": rhs.append(token[1:-1])
             #    else: rhs.append(Nonterminal(token))
             #    return ''
-            #CFGEditor._TOKEN_RE.sub(parse_token, rhs_str)
+            # CFGEditor._TOKEN_RE.sub(parse_token, rhs_str)
             #
-            #productions.append(Production(lhs, *rhs))
+            # productions.append(Production(lhs, *rhs))
 
         return productions
 
     def _destroy(self, *e):
-        if self._top is None: return
+        if self._top is None:
+            return
         self._top.destroy()
         self._top = None
 
@@ -495,23 +535,36 @@ class CFGEditor(object):
             self._set_cfg_callback(self._cfg)
 
     def _cancel(self, *e):
-        try: self._reset()
-        except: pass
+        try:
+            self._reset()
+        except:
+            pass
         self._destroy()
 
     def _help(self, *e):
         # The default font's not very legible; try using 'fixed' instead.
         try:
-            ShowText(self._parent, 'Help: Chart Parser Demo',
-                     (_CFGEditor_HELP).strip(), width=75, font='fixed')
+            ShowText(
+                self._parent,
+                'Help: Chart Parser Demo',
+                (_CFGEditor_HELP).strip(),
+                width=75,
+                font='fixed',
+            )
         except:
-            ShowText(self._parent, 'Help: Chart Parser Demo',
-                     (_CFGEditor_HELP).strip(), width=75)
+            ShowText(
+                self._parent,
+                'Help: Chart Parser Demo',
+                (_CFGEditor_HELP).strip(),
+                width=75,
+            )
+
 
 ######################################################################
 # New Demo (built tree based on cfg)
 ######################################################################
 
+
 class CFGDemo(object):
     def __init__(self, grammar, text):
         self._grammar = grammar
@@ -523,7 +576,7 @@ class CFGDemo(object):
 
         # Base font size
         self._size = IntVar(self._top)
-        self._size.set(12) # = medium
+        self._size.set(12)  # = medium
 
         # Set up the key bindings
         self._init_bindings(self._top)
@@ -537,16 +590,18 @@ class CFGDemo(object):
         self._init_treelet(frame1)
         self._init_workspace(self._top)
 
-    #//////////////////////////////////////////////////
+    # //////////////////////////////////////////////////
     # Initialization
-    #//////////////////////////////////////////////////
+    # //////////////////////////////////////////////////
 
     def _init_bindings(self, top):
         top.bind('<Control-q>', self.destroy)
 
-    def _init_menubar(self, parent): pass
+    def _init_menubar(self, parent):
+        pass
 
-    def _init_buttons(self, parent): pass
+    def _init_buttons(self, parent):
+        pass
 
     def _init_grammar(self, parent):
         self._prodlist = ProductionList(parent, self._grammar, width=20)
@@ -566,15 +621,15 @@ class CFGDemo(object):
         self._tree = None
         self.reset_workspace()
 
-    #//////////////////////////////////////////////////
+    # //////////////////////////////////////////////////
     # Workspace
-    #//////////////////////////////////////////////////
+    # //////////////////////////////////////////////////
 
     def reset_workspace(self):
         c = self._workspace.canvas()
         fontsize = int(self._size.get())
-        node_font = ('helvetica', -(fontsize+4), 'bold')
-        leaf_font = ('helvetica', -(fontsize+2))
+        node_font = ('helvetica', -(fontsize + 4), 'bold')
+        leaf_font = ('helvetica', -(fontsize + 2))
 
         # Remove the old tree
         if self._tree is not None:
@@ -590,71 +645,79 @@ class CFGDemo(object):
             leaves.append(TextWidget(c, word, font=leaf_font, draggable=1))
 
         # Put it all together into one tree
-        self._tree = TreeSegmentWidget(c, rootnode, leaves,
-                                       color='white')
+        self._tree = TreeSegmentWidget(c, rootnode, leaves, color='white')
 
         # Add it to the workspace.
         self._workspace.add_widget(self._tree)
 
         # Move the leaves to the bottom of the workspace.
-        for leaf in leaves: leaf.move(0,100)
+        for leaf in leaves:
+            leaf.move(0, 100)
 
-        #self._nodes = {start:1}
-        #self._leaves = dict([(l,1) for l in leaves])
+        # self._nodes = {start:1}
+        # self._leaves = dict([(l,1) for l in leaves])
 
     def workspace_markprod(self, production):
         pass
 
     def _markproduction(self, prod, tree=None):
-        if tree is None: tree = self._tree
-        for i in range(len(tree.subtrees())-len(prod.rhs())):
+        if tree is None:
+            tree = self._tree
+        for i in range(len(tree.subtrees()) - len(prod.rhs())):
             if tree['color', i] == 'white':
-                self._markproduction
+                self._markproduction  # FIXME: Is this necessary at all?
 
             for j, node in enumerate(prod.rhs()):
-                widget = tree.subtrees()[i+j]
-                if (isinstance(node, Nonterminal) and
-                    isinstance(widget, TreeSegmentWidget) and
-                    node.symbol == widget.label().text()):
-                    pass # matching nonterminal
-                elif (isinstance(node, compat.string_types) and
-                      isinstance(widget, TextWidget) and
-                      node == widget.text()):
-                    pass # matching nonterminal
-                else: break
+                widget = tree.subtrees()[i + j]
+                if (
+                    isinstance(node, Nonterminal)
+                    and isinstance(widget, TreeSegmentWidget)
+                    and node.symbol == widget.label().text()
+                ):
+                    pass  # matching nonterminal
+                elif (
+                    isinstance(node, string_types)
+                    and isinstance(widget, TextWidget)
+                    and node == widget.text()
+                ):
+                    pass  # matching nonterminal
+                else:
+                    break
             else:
                 # Everything matched!
                 print('MATCH AT', i)
 
-    #//////////////////////////////////////////////////
+    # //////////////////////////////////////////////////
     # Grammar
-    #//////////////////////////////////////////////////
+    # //////////////////////////////////////////////////
 
     def _selectprod_cb(self, production):
         canvas = self._treelet_canvas
 
         self._prodlist.highlight(production)
-        if self._treelet is not None: self._treelet.destroy()
+        if self._treelet is not None:
+            self._treelet.destroy()
 
         # Convert the production to a tree.
         rhs = production.rhs()
         for (i, elt) in enumerate(rhs):
-            if isinstance(elt, Nonterminal): elt = Tree(elt)
+            if isinstance(elt, Nonterminal):
+                elt = Tree(elt)
         tree = Tree(production.lhs().symbol(), *rhs)
 
         # Draw the tree in the treelet area.
         fontsize = int(self._size.get())
-        node_font = ('helvetica', -(fontsize+4), 'bold')
-        leaf_font = ('helvetica', -(fontsize+2))
-        self._treelet = tree_to_treesegment(canvas, tree,
-                                            node_font=node_font,
-                                            leaf_font=leaf_font)
+        node_font = ('helvetica', -(fontsize + 4), 'bold')
+        leaf_font = ('helvetica', -(fontsize + 2))
+        self._treelet = tree_to_treesegment(
+            canvas, tree, node_font=node_font, leaf_font=leaf_font
+        )
         self._treelet['draggable'] = 1
 
         # Center the treelet.
         (x1, y1, x2, y2) = self._treelet.bbox()
         w, h = int(canvas['width']), int(canvas['height'])
-        self._treelet.move((w-x1-x2)/2, (h-y1-y2)/2)
+        self._treelet.move((w - x1 - x2) / 2, (h - y1 - y2) / 2)
 
         # Mark the places where we can add it to the workspace.
         self._markproduction(production)
@@ -665,11 +728,12 @@ class CFGDemo(object):
     def mainloop(self, *args, **kwargs):
         self._top.mainloop(*args, **kwargs)
 
+
 def demo2():
     from nltk import Nonterminal, Production, CFG
+
     nonterminals = 'S VP NP PP P N Name V Det'
-    (S, VP, NP, PP, P, N, Name, V, Det) = [Nonterminal(s)
-                                           for s in nonterminals.split()]
+    (S, VP, NP, PP, P, N, Name, V, Det) = [Nonterminal(s) for s in nonterminals.split()]
     productions = (
         # Syntactic Productions
         Production(S, [NP, VP]),
@@ -680,34 +744,40 @@ def demo2():
         Production(VP, [V, NP]),
         Production(PP, [P, NP]),
         Production(PP, []),
-
         Production(PP, ['up', 'over', NP]),
-
         # Lexical Productions
-        Production(NP, ['I']),   Production(Det, ['the']),
-        Production(Det, ['a']),  Production(N, ['man']),
-        Production(V, ['saw']),  Production(P, ['in']),
-        Production(P, ['with']), Production(N, ['park']),
-        Production(N, ['dog']),  Production(N, ['statue']),
+        Production(NP, ['I']),
+        Production(Det, ['the']),
+        Production(Det, ['a']),
+        Production(N, ['man']),
+        Production(V, ['saw']),
+        Production(P, ['in']),
+        Production(P, ['with']),
+        Production(N, ['park']),
+        Production(N, ['dog']),
+        Production(N, ['statue']),
         Production(Det, ['my']),
-        )
+    )
     grammar = CFG(S, productions)
 
     text = 'I saw a man in the park'.split()
-    d=CFGDemo(grammar, text)
+    d = CFGDemo(grammar, text)
     d.mainloop()
 
+
 ######################################################################
 # Old Demo
 ######################################################################
 
+
 def demo():
     from nltk import Nonterminal, CFG
-    nonterminals = 'S VP NP PP P N Name V Det'
-    (S, VP, NP, PP, P, N, Name, V, Det) = [Nonterminal(s)
-                                           for s in nonterminals.split()]
 
-    grammar = CFG.fromstring("""
+    nonterminals = 'S VP NP PP P N Name V Det'
+    (S, VP, NP, PP, P, N, Name, V, Det) = [Nonterminal(s) for s in nonterminals.split()]
+
+    grammar = CFG.fromstring(
+        """
     S -> NP VP
     PP -> P NP
     NP -> Det N
@@ -727,19 +797,25 @@ def demo():
     P -> 'up'
     P -> 'over'
     P -> 'with'
-    """)
+    """
+    )
+
+    def cb(grammar):
+        print(grammar)
 
-    def cb(grammar): print(grammar)
     top = Tk()
     editor = CFGEditor(top, grammar, cb)
     Label(top, text='\nTesting CFG Editor\n').pack()
     Button(top, text='Quit', command=top.destroy).pack()
     top.mainloop()
 
+
 def demo3():
     from nltk import Production
-    (S, VP, NP, PP, P, N, Name, V, Det) = \
-        nonterminals('S, VP, NP, PP, P, N, Name, V, Det')
+
+    (S, VP, NP, PP, P, N, Name, V, Det) = nonterminals(
+        'S, VP, NP, PP, P, N, Name, V, Det'
+    )
 
     productions = (
         # Syntactic Productions
@@ -751,20 +827,26 @@ def demo3():
         Production(VP, [V, NP]),
         Production(PP, [P, NP]),
         Production(PP, []),
-
         Production(PP, ['up', 'over', NP]),
-
         # Lexical Productions
-        Production(NP, ['I']),   Production(Det, ['the']),
-        Production(Det, ['a']),  Production(N, ['man']),
-        Production(V, ['saw']),  Production(P, ['in']),
-        Production(P, ['with']), Production(N, ['park']),
-        Production(N, ['dog']),  Production(N, ['statue']),
+        Production(NP, ['I']),
+        Production(Det, ['the']),
+        Production(Det, ['a']),
+        Production(N, ['man']),
+        Production(V, ['saw']),
+        Production(P, ['in']),
+        Production(P, ['with']),
+        Production(N, ['park']),
+        Production(N, ['dog']),
+        Production(N, ['statue']),
         Production(Det, ['my']),
-        )
+    )
 
     t = Tk()
-    def destroy(e, t=t): t.destroy()
+
+    def destroy(e, t=t):
+        t.destroy()
+
     t.bind('q', destroy)
     p = ProductionList(t, productions)
     p.pack(expand=1, fill='both')
@@ -774,4 +856,6 @@ def demo3():
     p.mark(productions[2])
     p.mark(productions[8])
 
-if __name__ == '__main__': demo()
+
+if __name__ == '__main__':
+    demo()
