@@ -7,30 +7,26 @@ from nose import SkipTest
 from nltk import classify
 
 TRAIN = [
-     (dict(a=1,b=1,c=1), 'y'),
-     (dict(a=1,b=1,c=1), 'x'),
-     (dict(a=1,b=1,c=0), 'y'),
-     (dict(a=0,b=1,c=1), 'x'),
-     (dict(a=0,b=1,c=1), 'y'),
-     (dict(a=0,b=0,c=1), 'y'),
-     (dict(a=0,b=1,c=0), 'x'),
-     (dict(a=0,b=0,c=0), 'x'),
-     (dict(a=0,b=1,c=1), 'y'),
- ]
+    (dict(a=1, b=1, c=1), 'y'),
+    (dict(a=1, b=1, c=1), 'x'),
+    (dict(a=1, b=1, c=0), 'y'),
+    (dict(a=0, b=1, c=1), 'x'),
+    (dict(a=0, b=1, c=1), 'y'),
+    (dict(a=0, b=0, c=1), 'y'),
+    (dict(a=0, b=1, c=0), 'x'),
+    (dict(a=0, b=0, c=0), 'x'),
+    (dict(a=0, b=1, c=1), 'y'),
+]
 
 TEST = [
-     (dict(a=1,b=0,c=1)), # unseen
-     (dict(a=1,b=0,c=0)), # unseen
-     (dict(a=0,b=1,c=1)), # seen 3 times, labels=y,y,x
-     (dict(a=0,b=1,c=0)), # seen 1 time, label=x
+    (dict(a=1, b=0, c=1)),  # unseen
+    (dict(a=1, b=0, c=0)),  # unseen
+    (dict(a=0, b=1, c=1)),  # seen 3 times, labels=y,y,x
+    (dict(a=0, b=1, c=0)),  # seen 1 time, label=x
 ]
 
-RESULTS = [
-    (0.16,  0.84),
-    (0.46,  0.54),
-    (0.41,  0.59),
-    (0.76,  0.24),
-]
+RESULTS = [(0.16, 0.84), (0.46, 0.54), (0.41, 0.59), (0.76, 0.24)]
+
 
 def assert_classifier_correct(algorithm):
     try:
@@ -49,5 +45,6 @@ def assert_classifier_correct(algorithm):
 def test_megam():
     assert_classifier_correct('MEGAM')
 
+
 def test_tadm():
     assert_classifier_correct('TADM')
