@@ -2,6 +2,7 @@
 from __future__ import absolute_import, unicode_literals
 from nltk.tag import hmm
 
+
 def _wikipedia_example_hmm():
     # Example from wikipedia
     # (http://en.wikipedia.org/wiki/Forward%E2%80%93backward_algorithm)
@@ -14,7 +15,7 @@ def _wikipedia_example_hmm():
     pi = [0.5, 0.5]  # initial probabilities
 
     seq = ['umbrella', 'umbrella', 'no umbrella', 'umbrella', 'umbrella']
-    seq = list(zip(seq, [None]*len(seq)))
+    seq = list(zip(seq, [None] * len(seq)))
 
     model = hmm._create_hmm_tagger(states, symbols, A, B, pi)
     return model, states, symbols, seq
@@ -26,12 +27,9 @@ def test_forward_probability():
     # example from p. 385, Huang et al
     model, states, symbols = hmm._market_hmm_example()
     seq = [('up', None), ('up', None)]
-    expected = [
-        [0.35, 0.02, 0.09],
-        [0.1792, 0.0085, 0.0357]
-    ]
+    expected = [[0.35, 0.02, 0.09], [0.1792, 0.0085, 0.0357]]
 
-    fp = 2**model._forward_probability(seq)
+    fp = 2 ** model._forward_probability(seq)
 
     assert_array_almost_equal(fp, expected)
 
@@ -40,7 +38,7 @@ def test_forward_probability2():
     from numpy.testing import assert_array_almost_equal
 
     model, states, symbols, seq = _wikipedia_example_hmm()
-    fp = 2**model._forward_probability(seq)
+    fp = 2 ** model._forward_probability(seq)
 
     # examples in wikipedia are normalized
     fp = (fp.T / fp.sum(axis=1)).T
@@ -61,7 +59,7 @@ def test_backward_probability():
 
     model, states, symbols, seq = _wikipedia_example_hmm()
 
-    bp = 2**model._backward_probability(seq)
+    bp = 2 ** model._backward_probability(seq)
     # examples in wikipedia are normalized
 
     bp = (bp.T / bp.sum(axis=1)).T
@@ -82,6 +80,7 @@ def test_backward_probability():
 
 def setup_module(module):
     from nose import SkipTest
+
     try:
         import numpy
     except ImportError:
