@@ -1,6 +1,6 @@
 # Natural Language Toolkit: Texts
 #
-# Copyright (C) 2001-2015 NLTK Project
+# Copyright (C) 2001-2019 NLTK Project
 # Author: Steven Bird <stevenbird1@gmail.com>
 #         Edward Loper <edloper@gmail.com>
 # URL: <http://nltk.org/>
@@ -13,20 +13,30 @@ Functionality includes: concordancing, collocation discovery,
 regular expression search over tokenized strings, and
 distributional similarity.
 """
-from __future__ import print_function, division, unicode_literals
+from __future__ import print_function, division, unicode_literals, absolute_import
 
 from math import log
-from collections import defaultdict
+from collections import defaultdict, Counter, namedtuple
 from functools import reduce
-from itertools import islice
 import re
+import sys
 
-from nltk.probability import FreqDist, LidstoneProbDist
+from six import text_type
+
+from nltk.lm import MLE
+from nltk.lm.preprocessing import padded_everygram_pipeline
+from nltk.probability import FreqDist
 from nltk.probability import ConditionalFreqDist as CFD
 from nltk.util import tokenwrap, LazyConcatenation
 from nltk.metrics import f_measure, BigramAssocMeasures
 from nltk.collocations import BigramCollocationFinder
-from nltk.compat import python_2_unicode_compatible, text_type, Counter
+from nltk.compat import python_2_unicode_compatible
+from nltk.tokenize import sent_tokenize
+
+ConcordanceLine = namedtuple(
+    "ConcordanceLine",
+    ["left", "query", "right", "offset", "left_print", "right_print", "line"],
+)
 
 
 class ContextIndex(object):
@@ -36,14 +46,15 @@ class ContextIndex(object):
     in a fixed window around the word; but other definitions may also
     be used by providing a custom context function.
     """
+
     @staticmethod
     def _default_context(tokens, i):
         """One left token and one right token, normalized to lowercase"""
-        left = (tokens[i-1].lower() if i != 0 else '*START*')
-        right = (tokens[i+1].lower() if i != len(tokens) - 1 else '*END*')
+        left = tokens[i - 1].lower() if i != 0 else "*START*"
+        right = tokens[i + 1].lower() if i != len(tokens) - 1 else "*END*"
         return (left, right)
 
-    def __init__(self, tokens, context_func=None, filter=None, key=lambda x:x):
+    def __init__(self, tokens, context_func=None, filter=None, key=lambda x: x):
         self._key = key
         self._tokens = tokens
         if context_func:
@@ -52,10 +63,12 @@ class ContextIndex(object):
             self._context_func = self._default_context
         if filter:
             tokens = [t for t in tokens if filter(t)]
-        self._word_to_contexts = CFD((self._key(w), self._context_func(tokens, i))
-                                     for i, w in enumerate(tokens))
-        self._context_to_words = CFD((self._context_func(tokens, i), self._key(w))
-                                     for i, w in enumerate(tokens))
+        self._word_to_contexts = CFD(
+            (self._key(w), self._context_func(tokens, i)) for i, w in enumerate(tokens)
+        )
+        self._context_to_words = CFD(
+            (self._context_func(tokens, i), self._key(w)) for i, w in enumerate(tokens)
+        )
 
     def tokens(self):
         """
@@ -85,7 +98,9 @@ class ContextIndex(object):
         for c in self._word_to_contexts[self._key(word)]:
             for w in self._context_to_words[c]:
                 if w != word:
-                    scores[w] += self._context_to_words[c][word] * self._context_to_words[c][w]
+                    scores[w] += (
+                        self._context_to_words[c][word] * self._context_to_words[c][w]
+                    )
         return sorted(scores, key=scores.get, reverse=True)[:n]
 
     def common_contexts(self, words, fail_on_unknown=False):
@@ -104,24 +119,25 @@ class ContextIndex(object):
         empty = [words[i] for i in range(len(words)) if not contexts[i]]
         common = reduce(set.intersection, contexts)
         if empty and fail_on_unknown:
-            raise ValueError("The following word(s) were not found:",
-                             " ".join(words))
+            raise ValueError("The following word(s) were not found:", " ".join(words))
         elif not common:
             # nothing in common -- just return an empty freqdist.
             return FreqDist()
         else:
-            fd = FreqDist(c for w in words
-                          for c in self._word_to_contexts[w]
-                          if c in common)
+            fd = FreqDist(
+                c for w in words for c in self._word_to_contexts[w] if c in common
+            )
             return fd
 
+
 @python_2_unicode_compatible
 class ConcordanceIndex(object):
     """
     An index that can be used to look up the offset locations at which
     a given word occurs in a document.
     """
-    def __init__(self, tokens, key=lambda x:x):
+
+    def __init__(self, tokens, key=lambda x: x):
         """
         Construct a new concordance index.
 
@@ -141,9 +157,7 @@ class ConcordanceIndex(object):
         """Function mapping each token to an index key (or None)."""
 
         self._offsets = defaultdict(list)
-        """Dictionary mapping words (or keys) to lists of offset
-           indices."""
-
+        """Dictionary mapping words (or keys) to lists of offset indices."""
         # Initialize the index (self._offsets)
         for index, word in enumerate(tokens):
             word = self._key(word)
@@ -168,39 +182,67 @@ class ConcordanceIndex(object):
         return self._offsets[word]
 
     def __repr__(self):
-        return '<ConcordanceIndex for %d tokens (%d types)>' % (
-            len(self._tokens), len(self._offsets))
+        return "<ConcordanceIndex for %d tokens (%d types)>" % (
+            len(self._tokens),
+            len(self._offsets),
+        )
 
-    def print_concordance(self, word, width=75, lines=25):
+    def find_concordance(self, word, width=80):
         """
-        Print a concordance for ``word`` with the specified context window.
-
-        :param word: The target word
-        :type word: str
-        :param width: The width of each line, in characters (default=80)
-        :type width: int
-        :param lines: The number of lines to display (default=25)
-        :type lines: int
+        Find all concordance lines given the query word.
         """
         half_width = (width - len(word) - 2) // 2
-        context = width // 4 # approx number of words of context
+        context = width // 4  # approx number of words of context
 
+        # Find the instances of the word to create the ConcordanceLine
+        concordance_list = []
         offsets = self.offsets(word)
         if offsets:
-            lines = min(lines, len(offsets))
-            print("Displaying %s of %s matches:" % (lines, len(offsets)))
             for i in offsets:
-                if lines <= 0:
-                    break
-                left = (' ' * half_width +
-                        ' '.join(self._tokens[i-context:i]))
-                right = ' '.join(self._tokens[i+1:i+context])
-                left = left[-half_width:]
-                right = right[:half_width]
-                print(left, self._tokens[i], right)
-                lines -= 1
+                query_word = self._tokens[i]
+                # Find the context of query word.
+                left_context = self._tokens[max(0, i - context) : i]
+                right_context = self._tokens[i + 1 : i + context]
+                # Create the pretty lines with the query_word in the middle.
+                left_print = " ".join(left_context)[-half_width:]
+                right_print = " ".join(right_context)[:half_width]
+                # The WYSIWYG line of the concordance.
+                line_print = " ".join([left_print, query_word, right_print])
+                # Create the ConcordanceLine
+                concordance_line = ConcordanceLine(
+                    left_context,
+                    query_word,
+                    right_context,
+                    i,
+                    left_print,
+                    right_print,
+                    line_print,
+                )
+                concordance_list.append(concordance_line)
+        return concordance_list
+
+    def print_concordance(self, word, width=80, lines=25):
+        """
+        Print concordance lines given the query word.
+        :param word: The target word
+        :type word: str
+        :param lines: The number of lines to display (default=25)
+        :type lines: int
+        :param width: The width of each line, in characters (default=80)
+        :type width: int
+        :param save: The option to save the concordance.
+        :type save: bool
+        """
+        concordance_list = self.find_concordance(word, width=width)
+
+        if not concordance_list:
+            print("no matches")
         else:
-            print("No matches")
+            lines = min(lines, len(concordance_list))
+            print("Displaying {} of {} matches:".format(lines, len(concordance_list)))
+            for i, concordance_line in enumerate(concordance_list[:lines]):
+                print(concordance_line.line)
+
 
 class TokenSearcher(object):
     """
@@ -212,8 +254,9 @@ class TokenSearcher(object):
     brackets as non-capturing parentheses, in addition to matching the
     token boundaries; and to have ``'.'`` not match the angle brackets.
     """
+
     def __init__(self, tokens):
-        self._raw = ''.join('<'+w+'>' for w in tokens)
+        self._raw = "".join("<" + w + ">" for w in tokens)
 
     def findall(self, regexp):
         """
@@ -240,21 +283,21 @@ class TokenSearcher(object):
         :type regexp: str
         """
         # preprocess the regular expression
-        regexp = re.sub(r'\s', '', regexp)
-        regexp = re.sub(r'<', '(?:<(?:', regexp)
-        regexp = re.sub(r'>', ')>)', regexp)
-        regexp = re.sub(r'(?<!\\)\.', '[^>]', regexp)
+        regexp = re.sub(r"\s", "", regexp)
+        regexp = re.sub(r"<", "(?:<(?:", regexp)
+        regexp = re.sub(r">", ")>)", regexp)
+        regexp = re.sub(r"(?<!\\)\.", "[^>]", regexp)
 
         # perform the search
         hits = re.findall(regexp, self._raw)
 
         # Sanity check
         for h in hits:
-            if not h.startswith('<') and h.endswith('>'):
-                raise ValueError('Bad regexp for TokenSearcher.findall')
+            if not h.startswith("<") and h.endswith(">"):
+                raise ValueError("Bad regexp for TokenSearcher.findall")
 
         # postprocess the output
-        hits = [h[1:-1].split('><') for h in hits]
+        hits = [h[1:-1].split("><") for h in hits]
         return hits
 
 
@@ -278,6 +321,7 @@ class Text(object):
     >>> moby = Text(nltk.corpus.gutenberg.words('melville-moby_dick.txt'))
 
     """
+
     # This defeats lazy loading, but makes things faster.  This
     # *shouldn't* be necessary because the corpus view *should* be
     # doing intelligent caching, but without this it's running slow.
@@ -297,66 +341,109 @@ class Text(object):
 
         if name:
             self.name = name
-        elif ']' in tokens[:20]:
-            end = tokens[:20].index(']')
+        elif "]" in tokens[:20]:
+            end = tokens[:20].index("]")
             self.name = " ".join(text_type(tok) for tok in tokens[1:end])
         else:
             self.name = " ".join(text_type(tok) for tok in tokens[:8]) + "..."
 
-    #////////////////////////////////////////////////////////////
+    # ////////////////////////////////////////////////////////////
     # Support item & slice access
-    #////////////////////////////////////////////////////////////
+    # ////////////////////////////////////////////////////////////
 
     def __getitem__(self, i):
-        if isinstance(i, slice):
-            return self.tokens[i.start:i.stop]
-        else:
-            return self.tokens[i]
+        return self.tokens[i]
 
     def __len__(self):
         return len(self.tokens)
 
-    #////////////////////////////////////////////////////////////
+    # ////////////////////////////////////////////////////////////
     # Interactive console methods
-    #////////////////////////////////////////////////////////////
+    # ////////////////////////////////////////////////////////////
 
     def concordance(self, word, width=79, lines=25):
         """
-        Print a concordance for ``word`` with the specified context window.
+        Prints a concordance for ``word`` with the specified context window.
         Word matching is not case-sensitive.
+
+        :param word: The target word
+        :type word: str
+        :param width: The width of each line, in characters (default=80)
+        :type width: int
+        :param lines: The number of lines to display (default=25)
+        :type lines: int
+
         :seealso: ``ConcordanceIndex``
         """
-        if '_concordance_index' not in self.__dict__:
-            #print("Building index...")
-            self._concordance_index = ConcordanceIndex(self.tokens,
-                                                       key=lambda s:s.lower())
+        if "_concordance_index" not in self.__dict__:
+            self._concordance_index = ConcordanceIndex(
+                self.tokens, key=lambda s: s.lower()
+            )
 
-        self._concordance_index.print_concordance(word, width, lines)
+        return self._concordance_index.print_concordance(word, width, lines)
 
-    def collocations(self, num=20, window_size=2):
+    def concordance_list(self, word, width=79, lines=25):
         """
-        Print collocations derived from the text, ignoring stopwords.
+        Generate a concordance for ``word`` with the specified context window.
+        Word matching is not case-sensitive.
 
-        :seealso: find_collocations
-        :param num: The maximum number of collocations to print.
+        :param word: The target word
+        :type word: str
+        :param width: The width of each line, in characters (default=80)
+        :type width: int
+        :param lines: The number of lines to display (default=25)
+        :type lines: int
+
+        :seealso: ``ConcordanceIndex``
+        """
+        if "_concordance_index" not in self.__dict__:
+            self._concordance_index = ConcordanceIndex(
+                self.tokens, key=lambda s: s.lower()
+            )
+        return self._concordance_index.find_concordance(word, width)[:lines]
+
+    def collocation_list(self, num=20, window_size=2):
+        """
+        Return collocations derived from the text, ignoring stopwords.
+
+        :param num: The maximum number of collocations to return.
         :type num: int
         :param window_size: The number of tokens spanned by a collocation (default=2)
         :type window_size: int
         """
-        if not ('_collocations' in self.__dict__ and self._num == num and self._window_size == window_size):
+        if not (
+            "_collocations" in self.__dict__
+            and self._num == num
+            and self._window_size == window_size
+        ):
             self._num = num
             self._window_size = window_size
 
-            #print("Building collocations list")
+            # print("Building collocations list")
             from nltk.corpus import stopwords
-            ignored_words = stopwords.words('english')
+
+            ignored_words = stopwords.words("english")
             finder = BigramCollocationFinder.from_words(self.tokens, window_size)
             finder.apply_freq_filter(2)
             finder.apply_word_filter(lambda w: len(w) < 3 or w.lower() in ignored_words)
             bigram_measures = BigramAssocMeasures()
             self._collocations = finder.nbest(bigram_measures.likelihood_ratio, num)
-        colloc_strings = [w1+' '+w2 for w1, w2 in self._collocations]
-        print(tokenwrap(colloc_strings, separator="; "))
+        return [w1 + " " + w2 for w1, w2 in self._collocations]
+
+    def collocations(self, num=20, window_size=2):
+        """
+        Print collocations derived from the text, ignoring stopwords.
+
+        :param num: The maximum number of collocations to print.
+        :type num: int
+        :param window_size: The number of tokens spanned by a collocation (default=2)
+        :type window_size: int
+        """
+
+        collocation_strings = [
+            w1 + " " + w2 for w1, w2 in self.collocation_list(num, window_size)
+        ]
+        print(tokenwrap(collocation_strings, separator="; "))
 
     def count(self, word):
         """
@@ -385,41 +472,45 @@ class Text(object):
         :type num: int
         :seealso: ContextIndex.similar_words()
         """
-        if '_word_context_index' not in self.__dict__:
-            #print('Building word-context index...')
-            self._word_context_index = ContextIndex(self.tokens,
-                                                    filter=lambda x:x.isalpha(),
-                                                    key=lambda s:s.lower())
+        if "_word_context_index" not in self.__dict__:
+            # print('Building word-context index...')
+            self._word_context_index = ContextIndex(
+                self.tokens, filter=lambda x: x.isalpha(), key=lambda s: s.lower()
+            )
 
-#        words = self._word_context_index.similar_words(word, num)
+        # words = self._word_context_index.similar_words(word, num)
 
         word = word.lower()
         wci = self._word_context_index._word_to_contexts
         if word in wci.conditions():
             contexts = set(wci[word])
-            fd = Counter(w for w in wci.conditions() for c in wci[w]
-                          if c in contexts and not w == word)
+            fd = Counter(
+                w
+                for w in wci.conditions()
+                for c in wci[w]
+                if c in contexts and not w == word
+            )
             words = [w for w, _ in fd.most_common(num)]
             print(tokenwrap(words))
         else:
             print("No matches")
 
-
     def common_contexts(self, words, num=20):
         """
         Find contexts where the specified words appear; list
         most frequent common contexts first.
 
-        :param word: The word used to seed the similarity search
-        :type word: str
+        :param words: The words used to seed the similarity search
+        :type words: str
         :param num: The number of words to generate (default=20)
         :type num: int
         :seealso: ContextIndex.common_contexts()
         """
-        if '_word_context_index' not in self.__dict__:
-            #print('Building word-context index...')
-            self._word_context_index = ContextIndex(self.tokens,
-                                                    key=lambda s:s.lower())
+        if "_word_context_index" not in self.__dict__:
+            # print('Building word-context index...')
+            self._word_context_index = ContextIndex(
+                self.tokens, key=lambda s: s.lower()
+            )
 
         try:
             fd = self._word_context_index.common_contexts(words, True)
@@ -427,7 +518,7 @@ class Text(object):
                 print("No common contexts were found")
             else:
                 ranked_contexts = [w for w, _ in fd.most_common(num)]
-                print(tokenwrap(w1+"_"+w2 for w1,w2 in ranked_contexts))
+                print(tokenwrap(w1 + "_" + w2 for w1, w2 in ranked_contexts))
 
         except ValueError as e:
             print(e)
@@ -442,8 +533,62 @@ class Text(object):
         :seealso: nltk.draw.dispersion_plot()
         """
         from nltk.draw import dispersion_plot
+
         dispersion_plot(self, words)
 
+    def _train_default_ngram_lm(self, tokenized_sents, n=3):
+        train_data, padded_sents = padded_everygram_pipeline(n, tokenized_sents)
+        model = MLE(order=n)
+        model.fit(train_data, padded_sents)
+        return model
+
+    def generate(self, length=100, text_seed=None, random_seed=42):
+        """
+        Print random text, generated using a trigram language model.
+        See also `help(nltk.lm)`.
+
+        :param length: The length of text to generate (default=100)
+        :type length: int
+
+        :param text_seed: Generation can be conditioned on preceding context.
+        :type text_seed: list(str)
+
+        :param random_seed: A random seed or an instance of `random.Random`. If provided,
+        makes the random sampling part of generation reproducible. (default=42)
+        :type random_seed: int
+
+        """
+        # Create the model when using it the first time.
+        self._tokenized_sents = [
+            sent.split(" ") for sent in sent_tokenize(" ".join(self.tokens))
+        ]
+        if not hasattr(self, "trigram_model"):
+            print("Building ngram index...", file=sys.stderr)
+            self._trigram_model = self._train_default_ngram_lm(
+                self._tokenized_sents, n=3
+            )
+
+        generated_tokens = []
+
+        assert length > 0, "The `length` must be more than 0."
+        while len(generated_tokens) < length:
+            for idx, token in enumerate(
+                self._trigram_model.generate(
+                    length, text_seed=text_seed, random_seed=random_seed
+                )
+            ):
+                if token == "<s>":
+                    continue
+                if token == "</s>":
+                    break
+                generated_tokens.append(token)
+            random_seed += 1
+
+        prefix = " ".join(text_seed) + " " if text_seed else ""
+        output_str = prefix + tokenwrap(generated_tokens[:length])
+        print(output_str)
+        return output_str
+
     def plot(self, *args):
         """
         See documentation for FreqDist.plot()
@@ -456,7 +601,7 @@ class Text(object):
         :seealso: nltk.prob.FreqDist
         """
         if "_vocab" not in self.__dict__:
-            #print("Building vocabulary index...")
+            # print("Building vocabulary index...")
             self._vocab = FreqDist(self)
         return self._vocab
 
@@ -488,14 +633,15 @@ class Text(object):
             self._token_searcher = TokenSearcher(self)
 
         hits = self._token_searcher.findall(regexp)
-        hits = [' '.join(h) for h in hits]
+        hits = [" ".join(h) for h in hits]
         print(tokenwrap(hits, "; "))
 
-    #////////////////////////////////////////////////////////////
+    # ////////////////////////////////////////////////////////////
     # Helper Methods
-    #////////////////////////////////////////////////////////////
+    # ////////////////////////////////////////////////////////////
+
+    _CONTEXT_RE = re.compile("\w+|[\.\!\?]")
 
-    _CONTEXT_RE = re.compile('\w+|[\.\!\?]')
     def _context(self, tokens, i):
         """
         One left & one right token, both case-normalized.  Skip over
@@ -503,28 +649,28 @@ class Text(object):
         that is created for ``similar()`` and ``common_contexts()``.
         """
         # Left context
-        j = i-1
-        while j>=0 and not self._CONTEXT_RE.match(tokens[j]):
+        j = i - 1
+        while j >= 0 and not self._CONTEXT_RE.match(tokens[j]):
             j -= 1
-        left = (tokens[j] if j != 0 else '*START*')
+        left = tokens[j] if j != 0 else "*START*"
 
         # Right context
-        j = i+1
-        while j<len(tokens) and not self._CONTEXT_RE.match(tokens[j]):
+        j = i + 1
+        while j < len(tokens) and not self._CONTEXT_RE.match(tokens[j]):
             j += 1
-        right = (tokens[j] if j != len(tokens) else '*END*')
+        right = tokens[j] if j != len(tokens) else "*END*"
 
         return (left, right)
 
-    #////////////////////////////////////////////////////////////
+    # ////////////////////////////////////////////////////////////
     # String Display
-    #////////////////////////////////////////////////////////////
+    # ////////////////////////////////////////////////////////////
 
     def __str__(self):
-        return '<Text: %s>' % self.name
+        return "<Text: %s>" % self.name
 
     def __repr__(self):
-        return '<Text: %s>' % self.name
+        return "<Text: %s>" % self.name
 
 
 # Prototype only; this approach will be slow to load
@@ -544,19 +690,20 @@ class TextCollection(Text):
     Iterating over a TextCollection produces all the tokens of all the
     texts in order.
     """
-    def __init__(self, source, name=None):
-        if hasattr(source, 'words'): # bridge to the text corpus reader
+
+    def __init__(self, source):
+        if hasattr(source, "words"):  # bridge to the text corpus reader
             source = [source.words(f) for f in source.fileids()]
 
         self._texts = source
         Text.__init__(self, LazyConcatenation(source))
         self._idf_cache = {}
 
-    def tf(self, term, text, method=None):
+    def tf(self, term, text):
         """ The frequency of the term in text. """
         return text.count(term) / len(text)
 
-    def idf(self, term, method=None):
+    def idf(self, term):
         """ The number of texts in the corpus divided by the
         number of texts that the term appears in.
         If a term does not appear in the corpus, 0.0 is returned. """
@@ -564,33 +711,36 @@ class TextCollection(Text):
         idf = self._idf_cache.get(term)
         if idf is None:
             matches = len([True for text in self._texts if term in text])
-            # FIXME Should this raise some kind of error instead?
-            idf = (log(float(len(self._texts)) / matches) if matches else 0.0)
+            if len(self._texts) == 0:
+                raise ValueError("IDF undefined for empty document collection")
+            idf = log(len(self._texts) / matches) if matches else 0.0
             self._idf_cache[term] = idf
         return idf
 
     def tf_idf(self, term, text):
         return self.tf(term, text) * self.idf(term)
 
+
 def demo():
     from nltk.corpus import brown
-    text = Text(brown.words(categories='news'))
+
+    text = Text(brown.words(categories="news"))
     print(text)
     print()
     print("Concordance:")
-    text.concordance('news')
+    text.concordance("news")
     print()
     print("Distributionally similar words:")
-    text.similar('news')
+    text.similar("news")
     print()
     print("Collocations:")
     text.collocations()
     print()
-    #print("Automatically generated text:")
-    #text.generate()
-    #print()
+    # print("Automatically generated text:")
+    # text.generate()
+    # print()
     print("Dispersion plot:")
-    text.dispersion_plot(['news', 'report', 'said', 'announced'])
+    text.dispersion_plot(["news", "report", "said", "announced"])
     print()
     print("Vocabulary plot:")
     text.plot(50)
@@ -598,13 +748,16 @@ def demo():
     print("Indexing:")
     print("text[3]:", text[3])
     print("text[3:5]:", text[3:5])
-    print("text.vocab()['news']:", text.vocab()['news'])
+    print("text.vocab()['news']:", text.vocab()["news"])
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     demo()
 
-__all__ = ["ContextIndex",
-           "ConcordanceIndex",
-           "TokenSearcher",
-           "Text",
-           "TextCollection"]
+__all__ = [
+    "ContextIndex",
+    "ConcordanceIndex",
+    "TokenSearcher",
+    "Text",
+    "TextCollection",
+]
